%
% train SVM/SVM+/SVMMTL classifier 
%   input: 
%      data:  input data (multiple groups)
%      method: 'svm'     -- SVM 
%              'svmplus' -- SVM+(gamma)
%              'svmmtl'  -- SVM+MTL
%      para:  parameters   
%
%
function [classifier] = trainSVMplusMTL(data,method,para)

t = size(data,2); % number of groups  

% how many samples for each group ?
T = [];
X = [];
y = [];
pos = [0];
groupid =[];
epsilon = .00001;

kernel = para.kernel1;
kerneloption = para.kerneloption1;
kernel_kr = para.kernel2;
kerneloption_kr = para.kerneloption2;
C = para.C;

for r =1:t
    T=[T, size(data{r}.examples,1)];
    X= [X; data{r}.examples];
    y = [y; convert(data{r}.labels)];
    groupid = [groupid, ones(1,size(data{r}.examples,1))*r];
    pos = [pos, pos(r)+T(r)];
    Kr{r} = svmkernel(data{r}.examples,kernel_kr, kerneloption_kr);
end
L = sum(T);
Kz = svmkernel(X,kernel, kerneloption);
H = Kz.*(y*y')+eye(L)*epsilon;

P = zeros(L,L);
M = zeros(L,t);
for r=1:t
    P(pos(r)+1:pos(r+1),pos(r)+1:pos(r+1))= Kr{r};
    M(pos(r)+1:pos(r+1),r)=1;
end

switch method
    

    
case 'svm'  %SVM

cvx_begin
    variables a(L)
    minimize (0.5*(a'*H*a)-sum(a))
    y'*a ==0;
  %  M'*(a+m)== T'*C;
  %  P*(a+m)==C*P*ones(L,1);
    a>=0;
    a<=C;
  %  m>=0;
cvx_end
 
if isnan(a(1))
    b = nan;
    d(t) = 0;
else
cvx_begin
    variables b(1) epsino(L)
    minimize (sum(epsino))
    H*a+y*b>=1-epsino;
    epsino>=0;
cvx_end 
 
d(t) = 0; 
end

case 'svmplus' %SVM+gamma
    C =para.C;
    g =para.g;
    cvx_begin
        variables a(L) tmp(L) 
        maximize (sum(a)-0.5*a'*H*a -1/(2*g)*(tmp-C)'*P*(tmp-C))
        y'*a ==0;
        M'*(tmp)==T'*C;
        a>=0;
        tmp-a>=0;
    cvx_end
    
    cvx_begin
        variables b(1) d(t) 
        minimize (sum(d))
        H*a+y*b>=1-(1/g*P*(tmp-C)+M*d);
        1/g*P*(tmp-C)+M*d>=0;
    cvx_end
    
case 'svmmtl' %svm+MTL
    C =para.C;
    g =para.g;
    cvx_begin
        variables a(L) 
        maximize (sum(a)-0.5*a'*H*a -1/(2*g)*(a.*y)'*P*(a.*y))
        M'*(a.*y) ==0;
        a>=0;
        a<=C;
    cvx_end

  %  tmp2 = a/g;
    cvx_begin
        variables b(1) d(t) v(L)
        minimize (sum(v))
        H*a+y*b+y.*(P*((a/g).*y)+M*d)>=1-v;
        v>=0;
    cvx_end
    
    
   
end;
classifier.type = method;
classifier.y = y;
classifier.a = a;
classifier.xsup = X;
classifier.groupid = groupid;
classifier.dec_ker = para.kernel1;
classifier.dec_ker_opt = para.kerneloption1;
classifier.cor_ker = para.kernel2;
classifier.cor_ker_opt = para.kerneloption2;
classifier.b = b;
classifier.d = d;
classifier.training_parameter = para;




function y_new = convert(y)
% convert labels to -1,1
cond = unique(y);
y_new = zeros(size(y));
if size(cond,1)>1
    pos1 = find(y == cond(1));
    pos2 = find(y == cond(2));
    y_new(pos1) = -1;
    y_new(pos2) = 1;
else
    if cond==-1
        pos1 = find(y == cond(1));
        y_new(pos1) = -1;
    else
        pos1 = find(y == cond(1));
        y_new(pos1) = 1;
    end
end

